/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *
 *   Copyright (C) 2001-2006  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 *********************************************************************/

#include "constants.h"

#include "genscher.h"

#include "../../misc/translations.h"

/**
 ** constructor
 **
 ** @param	genscher	the player with the genscher
 ** @param	partner		the partner
 **
 ** @return	-
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
GameplayAction::Genscher::Genscher(unsigned const genscher,
				   unsigned const partner) :
  GameplayAction(GENSCHER, genscher),
  partner(partner)
{
} // Genscher::Genscher()

/**
 ** constructor
 **
 ** @param	line	line with the data
 **
 ** @return	-
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 ** 
 ** @todo	old format
 ** @todo	new format
 **/
GameplayAction::Genscher::Genscher(string line) :
  GameplayAction(GENSCHER, UINT_MAX),
  partner()
{
#ifndef OUTDATED
  // old format in the bug report:
  //   Played: Player 2: club ace
  if (string(line, 0, strlen("Genscher: ")) == "Genscher: ") {
    // ToDo

    return ;
  }
#endif

  // ToDo: new format

} // GameplayAction::Genscher(string line)

/**
 ** copy constructor
 **
 ** @param	genscher	action to copy
 **
 ** @return	action corresponding to the line
 **		(to be freed)
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
GameplayAction::Genscher::Genscher(Genscher const& genscher) :
  GameplayAction(genscher),
  partner(genscher.partner)
{
} // Genscher::Genscher(Genscher genscher)

/**
 ** copy constructor
 **
 ** @param	genscher	action to copy
 **
 ** @return	this action
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
GameplayAction::Genscher::Genscher&
GameplayAction::Genscher::operator=(Genscher const& genscher)
{
  this->GameplayAction::operator=(genscher);
  this->partner = genscher.partner;
  return *this;
} // Genscher& Genscher::operator=(Genscher genscher)

/**
 ** destructor
 **
 ** @param	-
 **
 ** @return	-
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
GameplayAction::Genscher::~Genscher()
{
} // Genscher::~Genscher()

/**
 **
 ** writes the action in the output stream
 **
 ** @param	ostr	output stream
 **
 ** @return	-
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
void
GameplayAction::Genscher::write(ostream& ostr) const
{
  ostr << this->type;
  ostr << " = " << this->player << ", " << this->partner;

  return ;
} // void write(ostream& ostr) const

/**
 ** -> result
 **
 ** @param	-
 **
 ** @return	a translation representing the data
 ** 
 ** @author	Diether Knof
 **
 ** @version	0.7.1
 **/
Translator::Translation
GameplayAction::Genscher::data_translation() const
{
  return ::translation("%uunsigned%", this->partner);
} // virtual Translator::Translation GameplayAction::Genscher::data_translation() const
