/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *
 *   Copyright (C) 2001-2006  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 *********************************************************************/

#include "constants.h"

#include "poverty_shift.h"

#include "../../misc/translations.h"

/**
 ** constructor
 **
 ** @param	playerno	player who has shifted the cards
 ** @param	cards		shifted cards
 **
 ** @return	-
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
GameplayAction::PovertyShift::PovertyShift(unsigned const playerno,
					   vector<Card> const& cards) :
  GameplayAction(POVERTY_SHIFT, playerno),
  cards(cards)
{
} // PovertyShift::PovertyShift(unsigned playerno, vector<Card> cards)

/**
 ** constructor
 **
 ** @param	line	line with the data
 **
 ** @return	action corresponding to the line
 **		(to be freed)
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 ** 
 ** @todo	old format
 ** @todo	new format
 **/
GameplayAction::PovertyShift::PovertyShift(string line) :
  GameplayAction(POVERTY_RETURNED, UINT_MAX),
  cards()
{
#ifndef OUTDATED
  // old format in the bug report:
  //   Played: Player 2: club ace
  if (string(line, 0, strlen("")) == "Poverty Shift: ") {
    // ToDo

    return ;
  }
#endif

  // ToDo: new format

} // GameplayAction::PovertyShift(string line)

/**
 ** copy constructor
 **
 ** @param	poverty_shift	action to copy
 **
 ** @return	-
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
GameplayAction::PovertyShift::PovertyShift(PovertyShift const&
					   poverty_shift) :
  GameplayAction(poverty_shift),
  cards(poverty_shift.cards)
{
} // PovertyShift::PovertyShift(PovertyShift poverty_shift)

/**
 ** copy constructor
 **
 ** @param	poverty_shift	action to copy
 **
 ** @return	this action
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
GameplayAction::PovertyShift::PovertyShift&
GameplayAction::PovertyShift::operator=(PovertyShift const& poverty_shift)
{
  this->GameplayAction::operator=(poverty_shift);
  this->cards = poverty_shift.cards;
  return *this;
} // PovertyShift& PovertyShift::operator=(PovertyShift poverty_shift)

/**
 ** destructor
 **
 ** @param	-
 **
 ** @return	-
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
GameplayAction::PovertyShift::~PovertyShift()
{
} // PovertyShift::~PovertyShift()

/**
 **
 ** writes the action in the output stream
 **
 ** @param	ostr	output stream
 **
 ** @return	-
 **
 ** @author	Diether Knof
 ** 
 ** @version	0.7.1
 **/
void
GameplayAction::PovertyShift::write(ostream& ostr) const
{
  ostr << this->type;
  ostr << " = " << this->player;
  for (vector<Card>::const_iterator c = this->cards.begin();
       c != this->cards.begin();
       ++c)
    ostr << " " << *c;

  return ;
} // void write(ostream& ostr) const

/**
 ** -> result
 **
 ** @param	-
 **
 ** @return	a translation representing the data
 ** 
 ** @author	Diether Knof
 **
 ** @version	0.7.1
 **/
Translator::Translation
GameplayAction::PovertyShift::data_translation() const
{
  vector<Card>::const_iterator c = this->cards.begin();
  if (c == this->cards.end())
    return ::translation("");

  Translator::Translation translation = ::translation(*c);
  for (++c; c != this->cards.begin(); ++c)
    translation += ", " + ::translation(*c);

  return translation;
} // virtual Translator::Translation GameplayAction::PovertyShift::data_translation() const
